#!/bin/sh
###########################################################BeginFile############
# Copyright:  (c) 2003 by Consul Risk Management B.V.  All Rights Reserved
#
# File-stamp: 
#
# Name:
#   getnewlogs.sh
#
# Purpose:
#   retrieve files modified since the previous run.
#   and store them sequentially in a new file
#
# Notes:
#   This script should be called from genact-main, since it depends on it to
#   set things up.
#   File names with spaces are supported, however should not be ever used.
#   
# History:
#   08may2001 - Created
#   26dec2003 AndrewZhdanov Revised:
#               Use 'regkey.propsfile' key for .ini properties and
#               Use functlib for logging and
#               Read log files in reversed order, i.e. latest last
###########################################################EndFile##############
# Source our function library. It contains generic functions.
. $CEA_GENACT_FUNCLIB || exit 1

# Compare two tokens
compv() {
  if [[ $1 > $2 ]] 
  then 
    echo "0"
  else
    echo "1"
  fi
}

# Convert month symbolic name to numeric value
mounth2num () {  
  case $1 in
    'Jan')
       outval="1"
       ;;
    'Fed')
       outval="2"
       ;;
    'Mar')
       outval="3"
       ;;
    'Apr')
       outval="4"
       ;;
    'May')
       outval="5"
       ;;
    'Jun')
       outval="6"
       ;;
    'Jul')
       outval="7"
       ;;
    'Aug')
       outval="8"
       ;;
    'Sep')
       outval="9"
       ;;
    'Oct')
       outval="a"
       ;;
    'Nov')
       outval="b"
       ;;
    'Dec')
       outval="c"
       ;;
     *)
       outval="0"
       ;;
  esac
  echo $outval
}

# Compare dates
# TODO: add description
cmpdate () {
  y1=`echo $2 | sed -e 's/^\([1-2][0-9][0-9][0-9]\).*/\1/'`
  M1=`echo $2 | sed -e 's/^[1-2][0-9][0-9][0-9] \([A-Z][a-z][a-z]\).*/\1/'`
  d1=`echo $2 | sed -e 's/^[1-2][0-9][0-9][0-9] [A-Z][a-z][a-z] \([0-3][0-9]\).*/\1/'`
  h1=`echo $2 | sed -e 's/^[1-2][0-9][0-9][0-9] [A-Z][a-z][a-z] [0-3][0-9] \([0-2][0-9]\).*/\1/'`
  m1=`echo $2 | sed -e 's/.*\([0-5][0-9]\)$/\1/'`
  M2=`echo $3 | sed -e 's/^\([A-Z][a-z][a-z]\).*/\1/'`
  d2=`echo $3 | sed -e 's/^[A-Z][a-z][a-z] \([0-3][0-9]\) .*/\1/'`
  d2=`echo $d2 | sed -e 's/^[A-Z][a-z][a-z] \([0-9]\) .*/0\1/'`
  M1=`mounth2num $M1`
  M2=`mounth2num $M2`
  if [ $1 = "1" ]
  then
    h2=`echo $3 | sed -e 's/^.*\([0-2][0-9]\):[0-5][0-9]$/\1/'`
    m2=`echo $3 | sed -e 's/^.*:\([0-5][0-9]\)$/\1/'`
    y2=`date +%Y`
    if [[ 'date +%m' < "07" ]]
    then
      if [[ "0$M2" < "06" ]]
      then
        y2=$(expr ${y2} - 1)
      fi
    fi
  else
    y2=`echo $3 | sed -e 's/^.*\([1-2][0-9][0-9][0-9]\)$/\1/'`
    h2='23'
    m2='59'
  fi
  if [ $y1 = $y2 ]
  then
    if [ $M1 = $M2 ]
    then
      if [ $d1 = $d2 ]
      then
        if [ $h1 = $h2 ]
        then
          echo `compv $m1 $m2`
        else
          echo `compv $h1 $h2`
        fi
      else
        echo `compv $d1 $d2`
      fi
    else
      echo `compv $M1 $M2`
    fi
  else
    echo `compv $y1 $y2`
  fi
}

# Validate input 

[[ $# -ge 1 ]] \
  || die 1 "Insufficient number of arguments."

[[ -n $OPTION2 && -n $OPTION3 && -n $OPTION4 ]] \
  || die 1 "Options 'Log Directory', 'Regexp' and  'Regkey' must be specified"

[[ -n $PROPSFILE ]] \
  || die 1 "Properties file must be specified"

filename=$1
directory=$OPTION2
regkey="$OPTION3.$PROPSFILE"
regexp="^$OPTION4\$"

debug "Initialization parameters:\n\t\t\tdirectory=$directory\n\t\t\tregkey=$regkey\n\t\t\tregexp=$regexp"

inifile="getnewlogs.ini"

# Initialize

if [ -e $inifile ]
then
  previous=`cat $inifile | grep "^$regkey:" | sed -e "s/^$regkey:\(.*\)$/\1/"`
fi

if [ -z $previous ]
then
  previous="1970 Jan 01 00 00"
fi

debug "Last date from previous run: $previous"

if [ -e $filename ]
then
  rm $filename
fi

if ! { touch $filename ; }
then
  die 2 "Can't create output file $filename"
fi

# Collect log files sorted by time stamp (latest last) created after previous run
filedate=""
flg="0"

ls=`ls -gotr $directory | grep "^-" | sed -e 's/^-[rwxs-][rwxs-]*  *[0-9][0-9]*  *[0-9][0-9]* \([A-Z][a-z][a-z]  *[0-9][0-9]*  *[0-9:][0-9:]*\) \(.*\)$/\1 \/ \2 \/\//'`

[[ -n $ls ]] || die 3 "Can't list directory $directory"

for aa in $ls
do
  if [ $flg = "1" ]
  then
    if [ $aa = "//" ]
    then 
      flg="0"
      if [ ! -z `echo "$fname" | grep "$regexp"` ]
      then
        if [ $fname != $filename ] 
        then
          debug "Read log file '$fname'"
          cat "$directory/$fname" >> $filename \
            || die 3 "Can't read logfile $fname"
        fi
      fi
    else
      if [ -z $fname ]
      then
        fname=$aa
      else
        fname="$fname $aa"
      fi
    fi
  else 
    case $aa in
     [A-Z][a-z][a-z])
       filedate=$aa
       ;;
     [0-3][0-9])
       filedate=$filedate" "$aa
       ;;
     [0-9])
       filedate=$filedate" 0"$aa
       ;;
     [0-2][0-9]:[0-5][0-9])
       filedate=$filedate" "$aa
       if [ `cmpdate 1 "$previous" "$filedate"` = "0" ]
       then
         break
       fi
       ;;
     [1-2][0-9][0-9][0-9])
       filedate=$filedate" "$aa
       if [ `cmpdate 2 "$previous" "$filedate"` = "0" ]
       then
         break
       fi
       ;;
     '/')
       flg="1"
       fname=""
       ;;
     *)
     filedate=""
     ;;
  esac
  fi
done

# Finalize

now=`date "+%Y %b %d %H %M"`

if ! { cat $inifile | grep -v "^$regkey:" > $inifile ; echo "$regkey:$now" >> $inifile ; }
then
  die 4 "Can't update inifile $inifile"
fi

debug "Finished successfully on $now"

exit 0
