#!/bin/sh
#***********************************************************BeginFile************
# Copyright:  (c) 2001 by Consul Risk Management B.V.  All Rights Reserved
# Time-stamp: <010924 Edwin 10:37 getnewrecs.sh>
# Name:
#   getnewrecs.sh
# Purpose:
#   retrieve records appended to a log since the previous run
#   and store them sequentially in a new file
# Notes:
#   getnewrecs.sh <outfilename> <platformname> <directory> <filename pattern> <LPI>
#   <outfilename>       Name of temp chunk To be collected
#  <platformname>      unique registry key value To store state of already collected Data
#   <directory>         path of directory To collect
#   <filename pattern>  regex of files To be collected from the directory
#   <Inifile name>      name of ini file to store state of collect		
# History:
#   010515 0.0.1 edwin EE105004: new
#   010528 0.0.10 edwin EE105004: added functionality to read current (open) logfiles as well as old
#                                 already closed files according To a filename pattern.
#   010809 0.0.16 Edwin QE108002: write first record if the logfile is switched
#   060121 0.0.85 Semechko WE05SA036 adjusting for Linux. fixing bugs
#   060213 0.0.93 Semechko WE05SA036 change name of inifile
#   060312 0.0.108 Semechko WE06S3001 adjusting for HP-UX, AIX, Solaris
#***********************************************************EndFile******

# 0 - no debug info (only critical messages)
# 1 - main functionality debug (should be default)
# 2 - deep debugging (input parameters of functions, return values)
# 3 - full debug
DEBUG_LEVEL="1"
DEBUG_LOG="../log/getnewrecs.log"

#
# Function: debug LEVEL DEBUG_MESSAGE
# Input: Level of message (see DEBUG_LEVEL variable)
#        Message to be outputed to debug file
# Return: none
#
debug ()
{
  test "$DEBUG_LEVEL" -lt "$1" && return
  echo "[`date`] $2" >> "$DEBUG_LOG"
}

#
# Function: isLesser YYYYMMDDhhmm YYYYMMDDhhmm
# Input: two timestamps to be compared in format YYYYMMDDhhmm
# Return: 0 - $1 lesser than $2; 1 - $1 is not lesser then $2 
# Note: this function is used instead of `test $1 -lt $2` 
# because on HP-UX `test` works wrong for large numbers
#
isLesser ()
{
  debug 2 "isLesser: [1] = [$1]"
  debug 2 "isLesser: [2] = [$2]"
  beg=1
  end=4
  while [ "$beg" -lt "10" ]
  do
    lval=`echo $1 | cut -c $beg-$end`
    rval=`echo $2 | cut -c $beg-$end`
    debug 3 "isLesser: beg = [$beg]; end = [$end]; lval = [$lval]; rval = [$rval]"
    test "$lval" -lt "$rval" && return 0
    test "$rval" -lt "$lval" && return 1
    debug 3 "isLesser: equal"
    beg=`expr $beg + 4`
    end=`expr $end + 4`
  done
  debug 3 "isLesser: fully equal values; return 1"
  return 1  
}

#
# Function: mton MONTH_STRING_VALUE
# Input: month as string
# Return: echoed month number
#
mton () 
{
  result="00"
  case "$1" in
    'Jan')
       result="01"
       ;;
    'Feb')
       result="02"
       ;;
    'Mar')
       result="03"
       ;;
    'Apr')
       result="04"
       ;;
    'May')
       result"05"
       ;;
    'Jun')
       result="06"
       ;;
    'Jul')
       result="07"
       ;;
    'Aug')
       result="08"
       ;;
    'Sep')
       result="09"
       ;;
    'Oct')
       result="10"
       ;;
    'Nov')
       result="11"
       ;;
    'Dec')
       result="12"
       ;;
     *)
       debug 0 "mton: unable to evaluate $1"
       ;;
  esac
  debug 2 "mton: result = [$result]"
  echo "$result"
}

#
# Function: dton DATE_OF_FILE
# Input: modtime of file as "ls -got" returned
#        <Month> <Day> <Hour>:<Min> 
#    or  <Month> <Day> <Year> in case file is more then one year old
# Return: echoed transformed timestamp in format YYYYMMDDhhmm
# 
dton ()
{
  debug 2 "dton: parameter #1 = [$1]"
  now=`date +%m%d%H%M`
  debug 3 "dton: now is [$now]"
  Year=""
  Hour=`echo $1 | grep [0-2][0-9]:[0-5][0-9] | sed -e 's/.\{1,\}\([0-2][0-9]\):[0-5][0-9]/\1/'`
  if [ -z "$Hour" ]
  then
    Hour="00"
    Year=`echo $1 | grep [1-2][0-9][0-9][0-9] | sed -e 's/.\{1,\}\([1-2][0-9][0-9][0-9]\)/\1/'`
  fi
  debug 3 "dton: Hour = [$Hour]"

  Min=`echo $1 | grep [0-2][0-9]:[0-5][0-9] | sed -e 's/.\{1,\}[0-2][0-9]:\([0-5][0-9]\)/\1/'`
  test -z "$Min" && Min="00"
  debug 3 "dton: Min = [$Min]"

  Month=`echo $1 | sed -e 's/.\{0,\}\([A-Z][a-z][a-z]\).\{1,\}/\1/'`
  Month=`mton $Month`
  debug 3 "dton: Month = [$Month]"

  Day=`echo $1 | grep " [0-3][0-9] " | sed -e 's/[A-Z][a-z][a-z] \{1,\}\([0-3][0-9]\).\{1,\}/\1/'`
  test -z "$Day" && Day=`echo $1 | sed -e 's/[A-Z][a-z][a-z] \{1,\}\([1-9]\).\{1,\}/0\1/'`
  debug 3 "dton: Day = [$Day]"

  if [ -z "$Year" ]
    then
    Year=`date +%Y`
    test "$now" -lt "$Month""01""$Hour""$Min" && Year=`expr $Year - 1`
      fi
  debug 3 "dton: Year = [$Year]"
  
  result="$Year""$Month""$Day""$Hour""$Min"
  debug 2 "dton: result = [$result]"
  echo "$result"
}


#
# MAIN
#
  
debug 0 "COLLECT STARTED, DEBUG_LEVEL = [$DEBUG_LEVEL]"

OutFileName="$1"
PltfName="$2"
# WE05SA036: Semechko: removing last '/' if it is not root directory
Directory=`echo "$3" | sed -e "s/\(.\{1,\}\)\/$/\1/"`   
FilenamePattern="$4"
Inifile="../run/$5"
debug 1 "main: OutFileName = [$OutFileName]" 
debug 1 "main: PltfName = [$PltfName]"
debug 1 "main: Directory = [$Directory]"
debug 1 "main: FilenamePattern = [$FilenamePattern]"
debug 1 "main: Inifile Name = [$Inifile]"

#especially here to see arguments in log in any case (required DEBUG_LEVEL>="1")
if [ "$#" -lt "5" ]
    then
  debug 0 "Insufficient number of arguments: $#; collect aborted. " 
  exit 1
      fi

previous="197001010000"
firstrecord="first run"
lastline=1

if [ -f "$Inifile" ]
  then
  debug 3 "main: [$Inifile] found"  
  iniprev=`cat $Inifile | grep "^$PltfName:"`
  if [ -n "$iniprev" ]
    then
#get timestamp from previous run1
    previous=`cat $Inifile | grep "^$PltfName:" | sed -e "s/^$PltfName:\([^:][^:]*\):.*$/\1/"`
# get last linenumber from previous run
    lastline=`cat $Inifile | grep "^$PltfName:" | sed -e "s/^$PltfName:[^:][^:]*:\([0-9][0-9]*\):.*$/\1/"`
#get first record from previous run
    firstrecord=`cat $Inifile | grep "^$PltfName:" | sed -e "s/^$PltfName:[^:][^:]*:[0-9][0-9]*:\(.*\)$/\1/"`
        fi
      else
  debug 3 "main: [$Inifile] not found"
  touch "$Inifile" && debug 3 "main: [$Inifile] created"
      fi

debug 1 "main: previous = [$previous]"
debug 1 "main: firstrecord = [$firstrecord]"
debug 1 "main: lastline = [$lastline]"

#open outfile
printf "" > "$OutFileName"

#get the filenames to process
#filenames can't consists spaces!!!!

buffer=""
fdate=""
fname=""
step=0

debug 1 "CREATING LIST OF FILES TO BE COLLECTED" 

for aa in `ls -got "$Directory" | grep "^-" | sed -e 's/.\{1,14\} \{1,\}[0-9]\{1,\} \{1,\}//'`
do
  debug 3 "main: step = [$step]; aa = [$aa]; fdate = [$fdate]"
  if [ "$step" -lt "3" ]
       then
    fdate="$fdate $aa"
    step=`expr $step + 1`
  else
    fname="$aa"
    fdate=`dton "$fdate"`
    debug 3 "main: fdate = [$fdate]; fname = [$fname]"
    isLesser "$fdate" "$previous" && break
    fname=`echo "$fname" | grep "$FilenamePattern"`
    debug 3 "main: fname = [$fname]"
    if [ -n "$fname" -a "$fname" != "$OutFileName" ]
    then
    buffer="$Directory/$fname $buffer"
# if here the first record of $fname is equal to $firstrecord then break
      frec=`head -n 1 $Directory/$fname`  
      test "$firstrecord" = "$frec" && break
    fi
    fdate=""
    step=0
       fi
done

debug 1 "main: buffer = [$buffer]"

#do the actual collection of the new records
for aa in $buffer
do
  debug 3 "main: file to be collected = [$aa]"
#compare if equal to line in registry (same file)
#start at first record if different
  record=`head -n 1 $aa`
  debug 3 "main: record = [$record]"
  if [ "$record" != "$firstrecord" ]
  then
    lastline="1"
    firstrecord="$record"
    head -n 1 "$aa" >> "$OutFileName"
  fi
  # WE05SA036: Semechko: obtain correct timestamp on diverse platforms
  tmpd=`ls -got "$aa" | sed -e "s/^.* \([A-Z][a-z][a-z] \{1,\}[0-9]\{1,\} \{1,\}[^ ]\{1,\}\) .*/\1/"`
  debug 3 "main: tmpd = [$tmpd]" 
  previous=`dton "$tmpd"`
  nline=`expr "$lastline" + 1`
  tail +"$nline" "$aa" >> "$OutFileName"
#get last linenumber of infile
  # WE05SA036: Semechko: obtain correct line number on diverse platforms
  lastline=`wc -l "$aa" | sed -e 's/^ *\([0-9]\{1,\}\).*/\1/'`
  debug 3 "main: lastline = [$lastline]"
done

#store new values 
cat "$Inifile" | grep -v "^$PltfName:" > "$Inifile.tmp" 
# cat is used because $firstrecord can contain '\' and we don't want to interpret them as escape characters
cat >> "$Inifile.tmp" <<EOF
$PltfName:$previous:$lastline:$firstrecord
EOF

mv "$Inifile.tmp" "$Inifile" 

