#!/bin/sh
# 0 - no debug info (only critical messages)
# 1 - main functionality debug (should be default)
# 2 - deep debugging (input parameters of functions, return values)
# 3 - full debug
DEBUG_LEVEL="1"
DEBUG_LOG="../log/wraparbitrary.log"

SYSLOG_NG_CONF=/usr/local/etc/syslog-ng/syslog-ng.conf

#
# Function: debug LEVEL DEBUG_MESSAGE
# Input: Level of message (see DEBUG_LEVEL variable)
#        Message to be outputed to debug file
# Return: none
#
debug ()
{
  test "$DEBUG_LEVEL" -lt "$1" && return
  echo "[`date`] $2" >> "$DEBUG_LOG"
}


#
# Function: die EXITCODE MESSAGE
# 
die ()
{
  debug 0 "ERROR: ""$2"
  debug 0 "Exit code: [$1]"
  exit $1
}


#
# MAIN
#

debug 0 "CHECKING CONFIGURATION, DEBUG_LEVEL = [$DEBUG_LEVEL]"
debug 1 "main: INSIGHT_SYSTEMNAME = [$INSIGHT_SYSTEMNAME]"
test -f "$SYSLOG_NG_CONF" || die 1 "file '$SYSLOG_NG_CONF' does not exists"

# check configuration
configured=0
beg=`grep -n "filter \{1,\}f_ism_hosts" "$SYSLOG_NG_CONF" | sed -e 's/\([0-9]\{1,\}\).\{1,\}/\1/'`
test -z "$beg" && die 2 "filter f_ism_hosts not found"
end=`tail -n +"$beg" "$SYSLOG_NG_CONF" | grep -nm 1 "};" | sed -e 's/\([0-9]\{1,\}\).\{1,\}/\1/'`
test -z "$end" && die 2 "filter f_ism_hosts terminated incorrectly; not closing '};' found"
end=`expr $beg + $end - 1`

debug 1 "main: filter 'f_ism_hosts' definition found on lines [$beg-$end]"

while [ "$beg" -le "$end" ]
do
  debug 2 "main: beg = [$beg]"

  line=`tail -n +"$beg" "$SYSLOG_NG_CONF" | head -n 1`

  debug 2 "main: line = [$line]"
  
  echo "$line" | grep "host(\"$INSIGHT_SYSTEMNAME\")" >> /dev/null 
  if [ "$?" -eq "0" ]
  then
    debug 2 "main: host $INSIGHT_SYSTEMNAME definition found in configuration file"
    configured=1
    break
  fi
  beg=`expr $beg + 1`
done

test "$configured" -eq "0" && die 3 "host is not configured to audit $INSIGHT_SYSTEMNAME"
debug 1 "main: host is configured to audit $INSIGHT_SYSTEMNAME"

# starting getnewrecs.sh
../bin/getnewrecs.sh "$1" "ArbitraryLinux" "$2/$INSIGHT_SYSTEMNAME" "$3" "$4"

