'***********************************************************
'* Copyright(C) 2001 CONSUL Risk Management
'*
'* Purpose: performs processing specific log file 
'*          that passed from the getdb2logs.bat script
'* 
'* Notes: 
'*        getdb2recs.vbs <db2_log_path> <log_file_name> <output_file> <props_file>
'*        <db2_log_path>   path to the audit log file
'*        <log_file_name>  name of the audit log file
'*        <output_file>    temporary file to store new logs
'*        <props_file>     actuator state file
'*
'* History:
'      20060106 Pipe-A EE0408009: created.
'      20060201 Pipe-A EE0408009: modified
'*
'***********************************************************

Option Explicit      ' Force explicit variable declaration.
On Error Resume Next
Err.Clear

' Variable declaration
Dim args             ' the arguments of this program
Dim wshShell         ' the shell to direct commands to

Dim db2_log_path     ' path to the audit log file
Dim log_file_name    ' name of the audit log file
Dim output_file      ' temporary file to store new logs
Dim props_file       ' actuator state file

Dim fso              ' file System Object

Dim progressFile     ' file to store actuator progress information
Dim auditlogFile     ' log_file_name file with audit records
Dim outputFile       ' output_file file with new audit records
Dim stateFileForAll  ' file to store actuator state information for all audited log files
Dim fileAccess       ' checks access for <stateFileForAll> file

Dim first_record     ' first record of audit log file
Dim last_record      ' last record of audit log file
Dim line_shift       ' number of lines between first and last records of audit log file

Dim logrecord        ' stores temporary log record
Dim num              ' stores temporary log record number

' Constant declaration
Const emptyfile = "Empty file"
Const notset   = "-"

Set wshShell = wscript.CreateObject("WScript.Shell")

'=======================================================================
'============< Checking for parameters >================================
'=======================================================================

' ***********************************************
' Step 1: Command Line Arguments processsing
' ***********************************************

Set args = wscript.Arguments

' Check whether appropriate number of arguments is specified
If args.Count <> 4 Then
  wscript.Echo "ERROR: insufficient number of arguments."
End If

db2_log_path = args(0)
log_file_name = args(1)
output_file = args(2)
props_file = args(3)

' ***********************************************
' End of Command Line Arguments processing
' ***********************************************

'=======================================================================
'============< Checking for existence and open text files >=============
'=======================================================================

Err.Clear

Set fso = CreateObject("Scripting.FileSystemObject")

' Initiate logging file to store actuator progress information
Set progressFile = fso.OpenTextFile("..\log\" & props_file & ".log", 8, True, 0)

' ***********************************************
' Step 2: If file <db2_log_path\log_file_name> doesn't exist, finish work with return code 1
' ***********************************************

If (fso.FolderExists(db2_log_path) = False) Then 
  Call progressFile.WriteLine("getdb2recs.vbs ERROR: The path to the DB2 UDB instance does not exist: " & db2_log_path)
  progressFile.Close
  Call wscript.Quit(1)
End If
If (fso.FileExists(db2_log_path & "\" & log_file_name) = False) Then 
  Call progressFile.WriteLine("getdb2recs.vbs ERROR: The audit log file <" & log_file_name & "> does not exist")
  progressFile.Close
  Call wscript.Quit(1)
End If

' Open file for reading
Err.Clear
Set auditlogFile = fso.OpenTextFile(db2_log_path & "\" & log_file_name, 1, False, 0)
If Err.Number <> 0 Then
  Call progressFile.WriteLine("getdb2recs.vbs ERROR: <" & log_file_name & "> could not be opened for reading. Error number - " & Err.Number & ", description - " & Err.Description)
  progressFile.Close
  Call wscript.Quit(1)
End If

' Open file for writing
Err.Clear
Set outputFile = fso.OpenTextFile(output_file, 2, True, 0)
If Err.Number <> 0 Then
  Call progressFile.WriteLine("getdb2recs.vbs ERROR: <" & output_file & "> could not be opened for writing. Error number - " & Err.Number & ", description - " & Err.Description)
  progressFile.Close
  auditlogFile.Close
  Call wscript.Quit(1)
End If

' ***********************************************
' Step 3: If file <props_file> doesn't exist then perform steps 4-6, else - step 7
' ***********************************************

If (fso.FileExists(props_file) = False) Then
  ' Collect and finish
  Call Steps_4_6()
End If

'=======================================================================
'============< start of MAIN >==========================================
'=======================================================================

' ***********************************************
' Step 7: If <props_file> contains the all needed strings perform step 8,
'         else - write error message to log file and perform steps 4-6
' ***********************************************

first_record = notset
last_record  = notset
logrecord    = notset
num = 0
line_shift = 0

    ' Obtain results of previous collect
    Call ObtainResultsOfPreviousCollect()

If (first_record <> notset And last_record <> notset And IsNumeric(line_shift) = True) Then

  ' ***********************************************
  ' If during previous collect audit file was empty, then 
  ' copy all records from <log_file_name> to <output_file> file
  ' ***********************************************

  If first_record = emptyfile Then
    If Not auditlogFile.AtEndOfStream Then
      first_record = auditlogFile.ReadLine
      last_record = first_record
      num = num + 1
      outputFile.WriteLine(first_record)
    Else
      first_record = emptyfile
      last_record = emptyfile
      line_shift = 0

      ' Finish
      Call Steps_5_6()
    End If

    Call CopyNewLogRecords()

    ' Finish
    Call Steps_5_6()
  End If

  ' ***********************************************
  ' Step 8: If text of the first record of <log_file_name> equal to <first_record> variable 
  '         from <props_file>, then perform step 9, else - step 10
  ' ***********************************************

  If Not auditlogFile.AtEndOfStream Then
    logrecord = auditlogFile.ReadLine
    num = num + 1
  Else
    first_record = emptyfile
    last_record = emptyfile
    line_shift = 0

    ' Finish
    Call Steps_5_6()
  End If

  If StrComp(first_record, logrecord, 0) = 0 Then

    ' ***********************************************
    ' Step 9: Copy all records except the first <line_shift> records
    '         from <log_file_name> to <output_file> file
    ' ***********************************************

    Dim i
    For i = 2 To line_shift ' start from second string
      If Not auditlogFile.AtEndOfStream Then
        auditlogFile.SkipLine
        num = num + 1
      Else
        Call progressFile.WriteLine("getdb2recs.vbs ERROR: number of records in <" & log_file_name & "> file is less then line shift in <" & props_file & "> property file")

        Call SetPointerToBeginningOfFile()
        Call CopyNewLogRecords()

        ' Finish
        Call Steps_5_6()
      End If
    Next

    Call CopyNewLogRecords()

    ' Finish
    Call Steps_5_6()

  Else

    first_record = logrecord
      
    ' ***********************************************
    ' Step 10: Find the record in <log_file_name> that is equal to <last_record> variable.
    '          If record was found, then perform step 11, else - step 12
    ' ***********************************************

    Do While StrComp(last_record, logrecord, 0) <> 0 And auditlogFile.AtEndOfStream <> True
      Err.Clear
      logrecord = auditlogFile.ReadLine
      If Err.Number <> 0 Then
        Call progressFile.WriteLine("getdb2recs.vbs ERROR: read file error. File <" & log_file_name & ">, line number " & num & ", rror number - " & Err.Number & ", description - " & Err.Description)
        Exit Do
      End If
      num = num + 1
    Loop  

    If (last_record = logrecord) Then

      ' ***********************************************
      ' Step 11: Copy all records after the record that is equal to <last_record> variable
      '          from <log_file_name> to <output_file> file
      ' ***********************************************

      Call CopyNewLogRecords()

      ' Finish
      Call Steps_5_6()
    Else

      ' ***********************************************
      ' Step 12: Copy all records from <log_file_name> to <output_file> file
      ' ***********************************************

      Call SetPointerToBeginningOfFile()
      Call CopyNewLogRecords()

      ' Finish
      Call Steps_5_6()
    End If
  End If

Else
  ' Write error message
  Call progressFile.WriteLine("getdb2recs.vbs ERROR: previous collect of <" & log_file_name & "> was incorrect")

  ' Collect and finish
  Call Steps_4_6()
End If

'=====================================================================
'============< end of MAIN >==========================================
'=====================================================================



'============< subroutines >==========================================

' ###############################################
' ObtainResultsOfPreviousCollect
' ###############################################

Sub ObtainResultsOfPreviousCollect ' Obtain results of previous collect from actuator property file

  Dim objRegEx ' an instance of RegExp object
  Dim Match
  Dim Matches
  Dim tempStr

  ' Open file for reading
  Err.Clear
  fileAccess = True
  Set stateFileForAll = fso.OpenTextFile(props_file, 1, False, 0)
  If Err.Number <> 0 Then
    Call progressFile.WriteLine("getdb2recs.vbs ERROR: <" & props_file & "> could not be opened for reading. Error number - " & Err.Number & ", description - " & Err.Description)
    fileAccess = False
  End If

  If fileAccess = True Then

    Set objRegEx = WScript.CreateObject("VBScript.RegExp") ' create instance of RegExp object
    objRegEx.Global = False ' find first match only

    Do While stateFileForAll.AtEndOfStream <> True
      Err.Clear
      tempStr = stateFileForAll.ReadLine
      If Err.Number <> 0 Then
        Call progressFile.WriteLine("getdb2recs.vbs ERROR: read file error. File <" & props_file & ">, line number " & num & ", rror number - " & Err.Number & ", description - " & Err.Description)
        Exit Do
      End If

      objRegEx.Pattern = "^" & log_file_name & "_firstrecord="
      If objRegEx.Test(tempStr) = True Then
        first_record = objRegEx.Replace(tempStr, "")
      End If

      objRegEx.Pattern = "^" & log_file_name & "_lastrecord="
      If objRegEx.Test(tempStr) = True Then
        last_record = objRegEx.Replace(tempStr, "")
      End If

      objRegEx.Pattern = "^" & log_file_name & "_lineshift="
      If objRegEx.Test(tempStr) = True Then
        line_shift = objRegEx.Replace(tempStr, "")
      End If
    Loop

    stateFileForAll.Close
    Set objRegEx = Nothing
  End If

End Sub

' ###############################################
' SetPointerToBeginningOfFile
' ###############################################

Sub SetPointerToBeginningOfFile ' Set pointer to the beginning of audit log file

  ' Set file pointer on the beginning of audit log file
  auditlogFile.Close
  ' Open file for reading
  Err.Clear
  Set auditlogFile = fso.OpenTextFile(db2_log_path & "\" & log_file_name, 1, False, 0)
  If Err.Number <> 0 Then
    Call progressFile.WriteLine("getdb2recs.vbs ERROR: <" & log_file_name & "> could not be opened for reading. Error number - " & Err.Number & ", description - " & Err.Description)
    outputFile.Close
    progressFile.Close
    Call wscript.Quit(1)
  End If
      
  ' Set beginning value for <num> variable
  num = 0
  
  If Not auditlogFile.AtEndOfStream Then
    first_record = auditlogFile.ReadLine
    last_record = first_record
    num = num + 1
    outputFile.WriteLine(first_record)
  Else
    first_record = emptyfile
    last_record = emptyfile
    line_shift = 0
    
    ' Finish
    Call Steps_5_6()
  End If
  
End Sub

' ###############################################
' CopyNewLogRecords
' ###############################################

Sub CopyNewLogRecords ' Copy records from audit to output file

  Dim newlogrecord
  newlogrecord = last_record

  ' Copy log records from current position to the end of file
  Do While auditlogFile.AtEndOfStream <> True
    Err.Clear
    newlogrecord = auditlogFile.ReadLine
    If Err.Number <> 0 Then
      Call progressFile.WriteLine("getdb2recs.vbs ERROR: read file error. File <" & log_file_name & ">, line number " & num & ", rror number - " & Err.Number & ", description - " & Err.Description)
      Exit Do
    End If
    num = num + 1
    outputFile.WriteLine(newlogrecord)
  Loop

  ' Set <last_record> variable to the text of the last record of audit log file
  last_record = newlogrecord
  ' Set <line_shift> variable to the number of records of audit log file
  line_shift = num

End Sub

' ###############################################
' GetDateTime(inputphrase)
' ###############################################

Function GetDateTime(inputphrase)

  Dim timestamp
  Dim yearStr
  Dim monthStr
  Dim dayStr
  Dim hourStr
  Dim minuteStr
  Dim secondStr

  Dim objRegEx ' an instance of RegExp object
  Dim Match
  Dim Matches
  GetDateTime = 0

  Set objRegEx = WScript.CreateObject("VBScript.RegExp") ' create instance of RegExp object

  ' Find timestamp by pattern: 2005-12-29-16.20.24.859001
  objRegEx.Global = False ' find first match only
  objRegEx.Pattern = "\d{4}-\d{2}-\d{2}-\d{2}.\d{2}.\d{2}.\d{6}"
  If objRegEx.Test(inputphrase) = True Then
    Set Matches = objRegEx.Execute(inputphrase)
    For Each Match in Matches 
      timestamp = Match.Value
    Next
    Set Matches = Nothing
  End If

  ' Find year, month, day, hour, minute, second from timestamp
  objRegEx.Global = True ' find multiple match
  objRegEx.Pattern = "\d{2}"
  If objRegEx.Test(timestamp) = True Then
    Dim j
    j = 0
    Set Matches = objRegEx.Execute(timestamp)
    For Each Match in Matches
      Select Case j
        Case 0, 1
          yearStr = yearStr & Match.Value
        Case 2
          monthStr = Match.Value
        Case 3
          dayStr = Match.Value
        Case 4
          hourStr = Match.Value
        Case 5
          minuteStr = Match.Value
        Case 6
          secondStr = Match.Value
      End Select
      j = j + 1
    Next
    Set Matches = Nothing
  Else
    Call progressFile.WriteLine("getdb2recs.vbs ERROR: file <" & log_file_name & ">, line " & num & "; time stamp is empty")
  End If

    ' Verify if obtained data is a real date and time
    If IsDate(monthStr & "/" & dayStr & "/" & yearStr & " " & hourStr & ":" & minuteStr & ":" & secondStr) = True And monthStr < "13" And dayStr < "32" And hourStr < "24" And minuteStr < "60" And secondStr < "60" Then
      GetDateTime = DateValue(monthStr & "/" & dayStr & "/" & yearStr) & " " & TimeValue(hourStr & ":" & minuteStr & ":" & secondStr)
    Else
      Call progressFile.WriteLine("getdb2recs.vbs ERROR: file <" & log_file_name & ">, line " & num & "; time stamp <" & timestamp & "> is not a date")
    End If
    
  Set objRegEx = Nothing

End Function

' ###############################################
' Steps_4_6
' ###############################################

Sub Steps_4_6

  Dim datetime
  Dim twoDaysAgo

  twoDaysAgo = DateAdd("d", -2, Now()) ' 2 days ago date

  ' ***********************************************
  ' Step 4
  ' ***********************************************

  Call progressFile.WriteLine("getdb2recs.vbs: make collect of audit records from 2 last elapsed days for <" & log_file_name & "> file")

  ' Set default values
  logrecord = emptyfile
  num = 0
  datetime = 0

  If Not auditlogFile.AtEndOfStream Then
    logrecord = auditlogFile.ReadLine
    num = num + 1
    ' Get date and time from log record (using function GetDateTime)
    datetime = GetDateTime(logrecord)
  End If

  ' Set <first_record> variable to the text of the first record of audited log file
  first_record = logrecord

  Do While auditlogFile.AtEndOfStream <> True And DateDiff("s", twoDaysAgo, datetime) < 0
    Err.Clear
    logrecord = auditlogFile.ReadLine
    If Err.Number <> 0 Then
      Call progressFile.WriteLine("getdb2recs.vbs ERROR: read file error. File <" & log_file_name & ">, line number " & num & ", rror number - " & Err.Number & ", description - " & Err.Description)
      Exit Do
    End If
    num = num + 1
    ' Get date and time from log record (using function GetDateTime)
    datetime = GetDateTime(logrecord)
  Loop

  If DateDiff("s", twoDaysAgo, datetime) > 0 Then
    ' Copy all records of 2 last elapsed days from audit log file to output file
    outputFile.WriteLine(logrecord)
  End If

  Do While auditlogFile.AtEndOfStream <> True
    Err.Clear
    logrecord = auditlogFile.ReadLine
    If Err.Number <> 0 Then
      Call progressFile.WriteLine("getdb2recs.vbs ERROR: read file error. File <" & log_file_name & ">, line number " & num & ", rror number - " & Err.Number & ", description - " & Err.Description)
      Exit Do
    End If
    num = num + 1
    ' Copy all records of 2 last elapsed days from audit log file to output file
    outputFile.WriteLine(logrecord)
  Loop

  ' Set <last_record> variable to the text of the last record of audit log file
  last_record = logrecord
  ' Set <line_shift> variable to the number of records of audit log file
  line_shift = num

  ' Finish
  Call Steps_5_6()
  
End Sub

' ###############################################
' Steps_5_6
' ###############################################

Sub Steps_5_6

  ' ***********************************************
  ' Step 5: Create file <props_file.log_file_name> to store actuator state information
  ' ***********************************************

  Dim stateFile    ' File to stare actuator state information

  Set stateFile = fso.OpenTextFile(props_file & "." & log_file_name, 2, True, 0)

  ' Write strings to actuator state file
  Call stateFile.WriteLine(log_file_name & "_firstrecord=" & first_record)
  Call stateFile.WriteLine(log_file_name & "_lineshift=" & line_shift)
  Call stateFile.WriteLine(log_file_name & "_lastrecord=" & last_record)

  ' ***********************************************
  ' Step 6: Finish work with return code 0
  ' ***********************************************

  Call progressFile.WriteLine("getdb2recs.vbs: finish processing of <" & log_file_name & "> file")

  auditlogFile.Close
  outputFile.Close
  stateFile.Close  
  progressFile.Close
  Set stateFile = Nothing

  Call wscript.Quit(0)

End Sub

'============< end of program > ======================================

