'***********************************************************BeginFile************
' Copyright:  (c) 1998 by Consul Risk Management B.V.  All Rights Reserved
' Time-stamp: <021002 ide 17:11 getnoteslog.vbs>
' Name:
'   getnoteslog.vbs
'
' Purpose:
'   retrieve files modified since the previous run.
'   and store them sequentially in a new file
'
' Notes:
'   getnoteslog.vbs <outfile> <mailoutfile> <adminoutputfile> <dummy3> <server> ...
'                       ... <databasefile> [<userid>] [<pwd>] <propsfile>
'
'   If <userid> and <pwd> are entered, then the script MUST be run at the 
'   Lotus Notes Server
'
' History:
'    000613 0.1 ide <ID>:
'    010412 <Version> louis EE010003: added second sublog for Mail view
'    010627 <Version> martijn <ID>: corrected, restructured.
'    021002 <version> ide     <ID>: Improved documentation and logging
'    040109 <version> AndrewZhdanov: Added support Admin4.nsf and improved diagnosability
'    040712 <version> AndrewZhdanov: Remove support for server (anonymous) connection and minor fixes
'***********************************************************EndFile******
Option Explicit
On Error Resume Next ' handle error with handleError Sub

Dim args             ' Arguments parsed
Dim OutFileName      ' Name of file to copy misc. events to
Dim OutFileNameMail  ' Name of file to copy mail events to
Dim OutFileNameAdmin ' Name of file to copy admin events to
Dim dummy3           ' dummy3 for unused argument
Dim nlServer         ' Name of the Lotus Notes Server
Dim GetDbnlServer    ' Real Servername   
Dim nlDatabaseFile   ' databasefile name of events (log.nsf)
Dim nlDbaseFileAdmin ' databasefile name of events (admin4..nsf)
Dim nlUserId         ' User used to log in to notes Server
Dim nlPwd            ' Password of nlUserId
Dim PropsFile        ' Unique Id of Event Source instance
Dim debugLogObject   ' DebugLog
Dim debugLogFile     ' Enable debuggin messages ?
Dim notesVersion     ' Notes Version number
Dim errorNum         ' Error number 
Dim errorDesc        ' Error Description
' Constants
Const separator = ";"
Const debugLog = "..\log\Domino_"
Const tracing  = False

' Command Line argument pasring
Set args = wscript.Arguments
OutFileName     = args(0)
OutFileNameMail = args(1)
OutFileNameAdmin= args(2)
dummy3          = args(3)
nlServer        = args(4)
nlDatabaseFile  = args(5)
nlUserId        = args(6)
nlPwd           = args(7)
nlDbaseFileAdmin= args(8)
PropsFile       = args(9)

' Adjust variables
If nlServer = " " Then nlServer = ""       
If nlPwd    = " " Then nlPwd = ""
If nlUserId = " " Then nlUserId = ""

Dim regkey, regkeyMail,regkeyAdmin, nlViewName, nlViewNameMail, nlViewNameAdmin
regkey = "HKLM\Software\CONSUL Risk Management\CEAActuator\" & "Domino_5\" & PropsFile &_
         "\" & nlServer & "-" & nlDatabaseFile
regkeyMail = "HKLM\Software\CONSUL Risk Management\CEAActuator\" & "Domino_5\" & PropsFile &_
         "\" & nlServer & "-" & nlDatabaseFile & "-Mail"

regkeyAdmin = "HKLM\Software\CONSUL Risk Management\CEAActuator\" & "Domino_5\" & PropsFile &_
         "\" & nlServer & "-" & nlDbaseFileAdmin

nlViewName = "Miscellaneous Events"
nlViewNameMail = "Mail Routing Events"
nlViewNameAdmin = "All Activity by Server"

' Initiate logging file and write arguments
Dim removedProps, append
removedProps = Replace(PropsFile, ".props", "")
append = debugLog & removedProps & ".log"
Set debugLogObject = CreateObject("Scripting.FileSystemObject")
Set debugLogFile   = debugLogObject.CreateTextFile(append, True)

If tracing Then
  wscript.echo "Opened log file " & debugLog
  Call infoMessage(debugLogFile, "Open Notes Logfile")
  Call infoMessage(debugLogFile, "OutFileName     = args(0) : " & OutFileName )
  Call infoMessage(debugLogFile, "OutFileNameMail = args(1) : " & OutFileNameMail)
  Call infoMessage(debugLogFile, "OutFileNameAdmin= args(2) : " & OutFileNameAdmin)
  Call infoMessage(debugLogFile, "nlDbaseFileAdmin= args(3) : " & nlDbaseFileAdmin)
  Call infoMessage(debugLogFile, "nlServer        = args(4) : " & nlServer )
  Call infoMessage(debugLogFile, "nlDatabaseFile  = args(5) : " & nlDatabaseFile )
  If nlUserId = "" Then 
    Call infoMessage(debugLogFile, "nlUserId        = args(6) : " & "<empty>" )
  Else
    Call infoMessage(debugLogFile, "nlUserId        = args(6) : " & nlUserId )
  End if
  If nlPwd = "" Then 
    Call infoMessage(debugLogFile, "nlPwd           = args(7) : " & "<empty>" )
  Else
    Call infoMessage(debugLogFile, "nlPwd           = args(7) : " & "****" )
  End If
  Call infoMessage(debugLogFile, "PropsFile       = args(8) : " & PropsFile )
End If

' Validate input
If nlPwd = "" Then
  Call handleError(-1, "Password can not be empty, as server connection is no longer supported")
End If

' Initiate wscript shell
Dim wshShell
Set wshShell = wscript.CreateObject("WScript.Shell")
errorNum  = Err.Number
errorDesc = Err.Description
Call handleError(errorNum, errorDesc )

'For upgrade, use non-Mail key if -Mail key does not exist
If wshShell.RegRead(regkeyMail) = "" Then
  wshShell.RegWrite regkeyMail, wshShell.RegRead(regkey)
End If
' no error handling needed
 

'Open a new Notes session and open database
On Error Resume Next
Call infoMessage(debugLogFile, "Creating Lotus.NotesSession Object")
Dim session, logDb, logDbAdmin
Set session = CreateObject("Lotus.NotesSession")
errorNum  = Err.Number
errorDesc = Err.Description
Call handleError(errorNum, errorDesc )
Call infoMessage(debugLogFile, "Lotus.NotesSession Object created.")

'Initializing Session
' For help, see Notes 5 Administration Help, item : "hosting java applets"
Call infoMessage(debugLogFile, "Initializing Session")
If nlUserId = "" Then
  Call infoMessage(debugLogFile, " - Initializing with current user ID")
  session.Initialize nlPwd
  
  ' Initialize: this method can be used on a computer with a Notes client or
  ' Domino server and bases the session on the current user ID. If a password
  ' is specified, it must match the user ID password. If a password is not
  ' specified, the user is prompted for a password as required and as the
  ' software permits. Since VBScript does not support prompting, you must
  ' supply the password or the user ID must not have one.
  
Else
  Call infoMessage(debugLogFile, " - Initializing with user " & nlUserId)
  session.InitializeUsingNotesUserName nlUserId, nlPwd
  
  ' InitializeUsingNotesUserName: this method can be used only on a computer
  ' with a Domino server. InitializeUsingNotesUserName looks the name up in the
  ' local Domino Directory and permits access to the local server depending on
  ' the "Server Access" and "COM Restrictions" settings. The password must match
  ' the Internet password associated with the name. If no name is specified,
  ' access is granted if the server permits Anonymous access.
  
End If
errorNum  = Err.Number
errorDesc = Err.Description
Call handleError(errorNum, errorDesc )
Call infoMessage(debugLogFile, " - Initialization succeeded")
' Opening database on Server
If nlServer = "Local" Then
  Call infoMessage(debugLogFile, " - Opening databases on local server")
  GetDbnlServer = ""
Else
  Call infoMessage(debugLogFile, " - Opening databases on server " & nlServer)
  GetDbnlServer = nlServer
End If  
Set logDb = session.GetDatabase(GetDbnlServer, nlDatabaseFile)
errorNum  = Err.Number
errorDesc = Err.Description
Call handleError(errorNum, errorDesc )
Call infoMessage(debugLogFile, "GetDatabase")
' The logDb.Open call throws an error if the DB is already open. Ignore this error
' and continue
Call infoMessage(debugLogFile, "Opening database " & nlDatabaseFile)
logDb.Open
If logDb.IsOpen Then
  ' Read Miscellaneous Events
  Call infoMessage(debugLogFile, "")
  Call infoMessage(debugLogFile, "* Reading Miscellaneous Events")
  GetLog logDb, OutFileName, nlViewName, regkey

  ' Read Mail Events
  Call infoMessage(debugLogFile, "")
  Call infoMessage(debugLogFile, "* Reading Mail Events")
  GetLog logDb, OutFileNameMail, nlViewNameMail, regkeyMail
Else
  Call infoMessage(debugLogFile, "* Database file not found")
End If

' Read Server Events
Call infoMessage(debugLogFile, "")
Call infoMessage(debugLogFile, "GetDatabase")
Err.Number = 0
Set logDbAdmin = session.GetDatabase(GetDbnlServer, nlDbaseFileAdmin)
errorNum  = Err.Number
If errorNum =0 Then
  Call infoMessage(debugLogFile, " - Opening database " & nlDbaseFileAdmin)
  logDbAdmin.Open
  If logDbAdmin.IsOpen Then
    Call infoMessage(debugLogFile, "* Reading Server Events")
    GetLogAdmin logDbAdmin, OutFileNameAdmin, nlViewNameAdmin, regkeyAdmin
  Else
    Call infoMessage(debugLogFile, "* Database file not found")
  End If
else
  Call infoMessage(debugLogFile, " - Can't get access to " & nlDbaseFileAdmin)
End If


' Close debugfile and exit
Call infoMessage(debugLogFile, "Prossesing Event completed successfully")
Call debugLogFile.Close()
wscript.quit



'*************************************************************************************
' Name: GetLog
' Purpose: Copy events from a view in an opened database to a file
' Parameters:
'   logDb: An open NotesDatabase
'   OutFileName:              Name of file to copy events to.
'   nlViewName:               Name of view within Lotus.
'   regkey:                   Registry subkey to store timestamp between runs
'**************************************************************************************
Sub GetLog(logDb, OutFileName, nlViewName, regkey)
  Dim LastStartTimePreviousRun, StartTime, LastStartTime, ProcessStartTime
  Dim fso, MyFile
  Dim AllDocsView, AllDocsViewNav, CurrentNavEntry, Doc, ProcessDoc
  Dim ProcessDocs(), LastDocs()
  Dim ServerInLog, EventString, iStart, iEnd, FinishTime
  Dim currentLine, previousLine

  Call infoMessage(debugLogFile, "Opening View " & nlViewName)
  
  'Initialize timestamp-keeping
  On Error Resume Next
  LastStartTimePreviousRun = wshShell.RegRead(regkey)
  On Error Goto 0
  If Len(LastStartTimePreviousRun) = 0 Then LastStartTimePreviousRun = "01/01/1970 00:00:00 AM"
  
  Call infoMessage(debugLogFile, "Last Collect started at " & LastStartTimePreviousRun)

  'Create the output file
  Call infoMessage(debugLogFile, "Creating outputfile " & OutFileName)
  Set fso = CreateObject("Scripting.FileSystemObject")
  Set MyFile = fso.CreateTextFile(OutFileName, True)
  Call infoMessage(debugLogFile, "- outputfile created ")

  'Open a new view with all documents
  Call infoMessage(debugLogFile, "Retrieving all documents in view")
  Set AllDocsView = logDb.GetView(nlViewName)
  If AllDocsView Is Nothing Then Call handleError(-1, "View not found")
  Set AllDocsViewNav = AllDocsView.CreateViewNav()
  Set CurrentNavEntry = AllDocsViewNav.GetFirstDocument
  Call infoMessage(debugLogFile, "- Retrieved first document")
  Do While Not (CurrentNavEntry Is Nothing)
    Set Doc = CurrentNavEntry.Document
    ' log entry in debugfile
    On Error Resume Next
    If tracing Then
       Call infoMessage(debugLogFile, "- Retrieved document " & Doc.NotesURL )
    End If
    On Error GoTo 0
    
    If Not (Doc Is Nothing) Then
      StartTime = Doc.GetFirstItem("StartTime").Text
      If CDate(StartTime) >= CDate(LastStartTimePreviousRun) Then
        'Do not process last document(s) because they may not be complete yet
        If CDate(StartTime) > CDate(LastStartTime) Then
          ProcessStartTime = LastStartTime
          CopyObjArray LastDocs, ProcessDocs
          LastStartTime = StartTime
          ReDim LastDocs(0)
          Set LastDocs(0) = Doc
        ElseIf CDate(StartTime) = CDate(LastStartTime) Then
          ReDim Preserve LastDocs(ArrayDim(LastDocs))
          Set LastDocs(ArrayDim(LastDocs)-1) = Doc
          Erase ProcessDocs
        Else 'CDate(StartTime) < CDate(LastStartTime)
          ProcessStartTime = StartTime
          ReDim ProcessDocs(0)
          Set ProcessDocs(0) = Doc
        End If
        If tracing Then
           Call infoMessage(debugLogFile, "- StartTime = " & StartTime)
        End If
        For Each ProcessDoc in ProcessDocs
          If tracing Then
            Call infoMessage(debugLogFile, "  - ProcessStartTime = " & ProcessStartTime)
          End If
          MyFile.WriteLine "[StartTime]" & ProcessStartTime
          ServerInLog = ProcessDoc.GetFirstItem("Server").Text
          MyFile.WriteLine "[ServerName]" & ServerInLog
          If tracing Then
            Call infoMessage(debugLogFile, "- ServerInLog = " & ServerInLog)
          End If
          'Write all fields in Event String
          On Error Resume Next
          EventString = ProcessDoc.GetFirstItem("Events").Text ' Lotus 5.x
          If Err.Number = 0 Then 
            iStart = 1
            iEnd = 0
            previousLine = ""
            Do While iStart <= Len(EventString)
              If vbLf = Mid(EventString,iStart,1) then
                iStart = iStart + 1
              End if
              iEnd = InStr(iStart, EventString, vbCr)
              If iEnd = 0 Then
                iEnd = Len(EventString) + 1
              End if
              If iEnd <= Len(EventString) + 1 Then
                currentLine = Mid(EventString, iStart, iEnd - iStart)
                If " " = Mid(EventString,iStart,1) Then
                  previousLine = previousLine & currentLine
                Else
                  MyFile.WriteLine "[Server]" & ServerInLog & "[Events]" & _
                        previousLine
                  previousLine = currentLine
                End If
              End If
              iStart = iEnd + 1
            Loop
            If Not (previousLine = "") Then
              MyFile.WriteLine "[Server]" & ServerInLog & "[Events]" & _
                   previousLine
            End If
          Else
            If tracing Then Call infoMessage(debugLogFile, "Can't read Events property. Try EventList. ")
            Err.Clear
            EventString = ProcessDoc.GetFirstItem("EventList").Text
            If Err.Number = 0 Then
              Dim Evnt
              Dim Evnts
              On Error Resume Next
              Evnts = Split(EventString,";",-1,1) 
              errorNum  = Err.Number
              errorDesc = Err.Description
              Call handleError(errorNum, errorDesc )
              For Each Evnt in Evnts
                 MyFile.WriteLine "[Server]" & ServerInLog & "[Events]" & Evnt
              Next
            Else
              If tracing Then Call infoMessage(debugLogFile, "Can't read EventList property either")
              Call handleError(-1, "Can't read events from the Lotus document")
            End If
          End If
          FinishTime = ""
          FinishTime = ProcessDoc.GetFirstItem("FinishTime").Text
          On Error GoTo 0
          MyFile.WriteLine "[FinishTime]" & FinishTime
          MyFile.WriteLine "[UpdatedBy]" & ProcessDoc.GetFirstItem("$UpdatedBy").Text                 
        Next
      End If
    Else
      Call infoMessage(debugLogFile, "  - Document is empty" )
    End If
   'Go to next document, if any      
    If Doc Is Nothing Then
      Set CurrentNavEntry = Nothing
    Else
      Set CurrentNavEntry = AllDocsViewNav.GetNextDocument(CurrentNavEntry)
    End If
  Loop
  Call infoMessage(debugLogFile, "- All Docs Processed, last time = " & LastStartTime)
  MyFile.Close
  wshShell.RegWrite regkey, LastStartTime
End Sub
'*************************************************************************************
' Name: GetLogAdmin
' Purpose: Copy Admin events from a view in an opened database to a file
' Parameters:
'   logDb: An open NotesDatabase
'   OutFileName:              Name of file to copy events to.
'   nlViewName:               Name of view within Lotus.
'   regkey:                   Registry subkey to store timestamp between runs
'**************************************************************************************
Sub GetLogAdmin(logDb, OutFileName, nlViewName, regkey)
  Dim LastStartTimePreviousRun, StartTime, LastStartTime, ProcessStartTime
  Dim fso, MyFile
  Dim AllDocsView, AllDocsViewNav, CurrentNavEntry, Doc, ProcessDoc
  Dim ProcessDocs(), LastDocs()
  Dim ServerInLog, EventString, iStart, iEnd, FinishTime
  Dim currentLine, previousLine

  Call infoMessage(debugLogFile, "Opening View " & nlViewName)
  
  'Initialize timestamp-keeping
  On Error Resume Next
  LastStartTimePreviousRun = wshShell.RegRead(regkey)
  On Error Goto 0
  If Len(LastStartTimePreviousRun) = 0 Then LastStartTimePreviousRun = "01/01/1970 00:00:00 AM"
  
  Call infoMessage(debugLogFile, "Last Collect started at " & LastStartTimePreviousRun)

  'Create the output file
  Call infoMessage(debugLogFile, "Creating outputfile " & OutFileName)
  Set fso = CreateObject("Scripting.FileSystemObject")
  Set MyFile = fso.CreateTextFile(OutFileName, True)
  Call infoMessage(debugLogFile, "- outputfile created ")

  'Open a new view with all documents
  Call infoMessage(debugLogFile, "Retrieving all documents in view")
  Set AllDocsView = logDb.GetView(nlViewName)
  If AllDocsView Is Nothing Then Call handleError(-1, "View not found")
  Set AllDocsViewNav = AllDocsView.CreateViewNav()
  Set CurrentNavEntry = AllDocsViewNav.GetFirstDocument
  Call infoMessage(debugLogFile, "- Retrieved first document")
  MyFile.Write "Server" & separator & "Date" & separator & "Action" & separator &_ 
  "ActionRequestor" & separator & "NameList" & separator & "Comments" & separator & "ErrorCode" 
  MyFile.WriteLine
  Do While Not (CurrentNavEntry Is Nothing)
    Set Doc = CurrentNavEntry.Document
    ' log entry in debugfile
    On Error Resume Next
    If tracing Then
      Call infoMessage(debugLogFile, "- Retrieved document " & Doc.NotesURL )
    End If
    On Error GoTo 0
    
    If Not (Doc Is Nothing) Then
           StartTime = Doc.Created
        If CDate(StartTime) >= CDate(LastStartTimePreviousRun) Then
         ProcessStartTime = StartTime
         ReDim ProcessDocs(0)
         Set ProcessDocs(0) = Doc
         If tracing Then
           Call infoMessage(debugLogFile, "- StartTime = " & StartTime)
         End If
         For Each ProcessDoc in ProcessDocs
          If tracing Then
            Call infoMessage(debugLogFile, "  - ProcessStartTime = " & ProcessStartTime)
          End If
          If Not ProcessDoc.HasItem("ProxyServerName") Then
            Call infoMessage(debugLogFile, "ERROR: There is no ServerName! It will be skipped!! ")
            Exit For
          End If
          If Not ProcessDoc.HasItem("ProxyActionRequestor") Then
            Call infoMessage(debugLogFile, "ERROR: There is no ProxyActionRequestor! It will be skipped!! ")
            Exit For
          End If

          If Not ProcessDoc.HasItem("ProxyAction") Then
            Call infoMessage(debugLogFile, "ERROR: There is no ProxyAction!!! It will be skipped!! ")
            Exit For
          End If

          If Not ProcessDoc.HasItem("ProxyNameList") Then
            Call infoMessage(debugLogFile, "ERROR: There is no ProxyNameList!!! It will be skipped!! ")
            Exit For
          End If
          ServerInLog = Replace(ProcessDoc.GetFirstItem("ProxyServerName").Text,separator,"")
          MyFile.Write ServerInLog
          MyFile.Write separator
          If tracing Then
            Call infoMessage(debugLogFile, "- ServerInLog = " & ServerInLog)
          End If
          MyFile.Write ProcessDoc.Created
          MyFile.Write separator
          MyFile.Write Replace(ProcessDoc.GetFirstItem("ProxyAction").Text,separator,"")
          MyFile.Write separator
          MyFile.Write Replace(ProcessDoc.GetFirstItem("ProxyActionRequestor").Text,separator,"")
          MyFile.Write separator
          MyFile.Write Replace(ProcessDoc.GetFirstItem("ProxyNameList").Text,separator,"")
          MyFile.Write separator
          If Doc.HasItem("ActionComments") Then
           MyFile.Write Replace(ProcessDoc.GetFirstItem("ActionComments").Text,separator,"")
          Else If Doc.HasItem("ProxyStatus") Then
           MyFile.Write Replace(ProcessDoc.GetFirstItem("ProxyStatus").Text,separator,"")
          Else
           MyFile.Write "None"
          End If
          End If
          MyFile.Write separator
          If Doc.HasItem("ErrorFlag") Then
           MyFile.Write Replace(ProcessDoc.GetFirstItem("ErrorFlag").Text,separator,"")
          Else
           MyFile.Write "0"
          End If

          MyFile.WriteLine
          If tracing Then
            Call infoMessage(debugLogFile, "Done!")
          End If
        Next
      Else
        if tracing then
          Call infoMessage(debugLogFile, "  - Was collected,skipped!" )
        end if
      End If
    Else
      Call infoMessage(debugLogFile, "  - Document is empty" )
    End If
   'Go to next document, if any      
    If Doc Is Nothing Then
      Set CurrentNavEntry = Nothing
    Else
      Set CurrentNavEntry = AllDocsViewNav.GetNextDocument(CurrentNavEntry)
    End If
   
  Loop
  Call infoMessage(debugLogFile, "- All Docs Processed, last time = " & CDate(now))
  MyFile.Close
  wshShell.RegWrite regkeyAdmin, CDate(now)
End Sub

'***************************************************
' Name: ArrayDim
' Purpose: Get the number of elements in an array
'***************************************************
Function ArrayDim(Arr)
  Dim Count
  On Error Resume Next
  Count = UBound(Arr) + 1
  ArrayDim = Count
End Function  

'****************************************************
' Name: CopyObjArray
' Purpose: Make a copy of an array containing objects
'****************************************************
Sub CopyObjArray(SourceArr, TargetArr)
  Dim Count, Index
  Count = ArrayDim(SourceArr)
  If Count = 0 Then
    Erase TargetArr
  Else
    ReDim TargetArr(Count - 1)
    For Index = 0 To Count - 1
      Set TargetArr(Index) = SourceArr(Index)
    Next
  End If
End Sub

'============================
'
' infoMessage()
'
Sub infoMessage(outputFile, text)
  If tracing Then
    wscript.echo "Message: " & text
  End If

  Call outputFile.WriteLine(Now & " " & text)
End Sub

'============================
'
' handleError()
'
Sub handleError(errorNum, errorDesc)
  If errorNum =0 Then
    'None
  Else
    Call infoMessage(debugLogFile, "ERROR Occured : " & errorDesc)
    wscript.quit errorNum
  End If
End Sub





