'On Error Resume Next
'***********************************************************BeginFile************
' Copyright:  (c) 1998 by Consul Risk Management B.V.  All Rights Reserved
' Time-stamp: <010227 ed 19:46 geteTrustlogs.vbs>
' Name:
'   geteTrustlogs.vbs
' Purpose:
'   retrieve files modified since the previous run.
'   and store them sequentially in a new file
' Notes:
'   geteTrustlogs.vbs <outfilename> <dummy> <dummy> <dummy> <SeAuditTool> 
'   <registry key> is used to store the date of the run
' History:
'    001124 <version> IN
'    2004oct27 0.2 VL: WE0407002: VBS actuators do not return exit codes - fixed
'***********************************************************EndFile******
Option Explicit      ' Force explicit variable declaration.
On Error Resume Next

' Variable declaration
Dim args             ' The arguments of this program
Dim OutFileName      ' The file containing the collected log data
Dim returnCode       ' Result of a command
Dim debugLogObject
Dim debugLogFile
Dim WshShell         ' The shell to direct commands to
Dim iCnt             ' Counter for arguments
Dim SeAuditTool      ' Full Pathname of the SeAudit Tool
Dim TimePeriod       ' Timeperiod for which to collect the logs
Dim TimePeriod1       ' Timeperiod for which to collect the logs (first day)
Dim TimePeriod2       ' Timeperiod for which to collect the logs 
Dim TimePeriod3      ' Timeperiod for which to collect the logs (last day)
Dim OutPutcatcher    ' redirection statement for output
Dim OutPutcatcher2   ' redirection statement for additional output
Dim CmdLine          ' The Commandline to be executed
Dim previous         ' Timestamp of Previous run ( in string format )
Dim d                ' Timestamp of Previous run ( in date format )
Dim enddatetime      ' Timestamp of this run
Dim sd               ' Date of Previous run ( in dd-mmm-yyyy format )
Dim ed               ' Date of Previous run ( in dd-mmm-yyyy format )
Dim st               ' Time of Previous run ( in 24-hrs format )
Dim et               ' Time of Previous run ( in 24-hrs format )
Err.Clear

Set wshShell = wscript.CreateObject("WScript.Shell")

' Constant declaration
Const regKey = "HKLM\Software\CONSUL Risk Management\CEAActuator\ETrust\ETrust"
Const ConstCmdSwitch = " -a -c -delim @@ "
Const debugLog = "..\log\ETrustAct.Log"
Const tracing  = False 'True 

' Initiate logging file
Set debugLogObject = CreateObject("Scripting.FileSystemObject")
Set debugLogFile   = debugLogObject.CreateTextFile(debugLog, True)

If Err.Number = 0 Then
	If tracing Then
		wscript.echo "Opened log file " & debugLog
	End If
Else
	If tracing Then
		wscript.echo "Failed to open log file " & debugLog
	End If
	Call CleanExit(1)
End If

Set args = wscript.Arguments
If tracing Then
  For iCnt = 0 To args.Count-1
    Call infoMessage(debugLogFile, "Args " & iCnt & " : " & args(iCnt))
  Next
End If

If args.Count < 5 Then
	Call Debug("Too few arguments!")
	Call CleanExit(1)
End If

OutFileName = args(0)
SeAuditTool = "cmd /c" + """" & args(4) & """"

previous = wshShell.RegRead(regKey)
If Err.Number <> 0 Then
    previous = "01/01/1970 00:01"
    Call Debug("Unable to get previous collect time, using " & previous)
End If

d = CDate(previous)
Call getDate(d,sd)
Call getTime(d,st)

'enddatetime = Now
enddatetime = DateAdd("n",-1,Now) 'subtract one minute from current Time
Call getDate(enddatetime,ed)
Call getTime(enddatetime,et)

OutPutcatcher = " > " + OutFileName
OutPutcatcher2 = " >> " + OutFileName

Dim start1,start2,start3
Dim stop1,stop2,stop3
Dim ed_minus_one,sd_plus_one

Err.Clear

' if everything happens on one day, run seaudit once
If ed = sd Then

  TimePeriod = " -sd " + sd + _
               " -st " + st + _
               " -ed " + ed + _
               " -et " + et

  Err.Clear
  CmdLine = SeAuditTool + ConstCmdSwitch + TimePeriod + OutPutcatcher
  Call ExecTool(CmdLine)
Else
  Dim date
  date=DateAdd("d",-1,ed)
  Call getDate(date,ed_minus_one)
  Call Debug("End date minus one day" + ed_minus_one)
  date=DateAdd("d",1,sd)
  Call getDate(date,sd_plus_one)
  Call Debug("Start date plus one day "+ sd_plus_one)
  'first day
  start1=" -sd "+sd+" -st "+st
  stop1=" -ed "+sd
  'last day
  start3=" -sd "+ed
  stop3=" -ed "+ed+" -et "+et
  ' if startdate+1==enddate we need to run seaudit twice
  If sd_plus_one = ed Then
    CmdLine = SeAuditTool + ConstCmdSwitch + start1+stop1 + OutPutcatcher
    Call ExecTool(CmdLine)
    CmdLine = SeAuditTool + ConstCmdSwitch + start3+stop3 + OutPutcatcher2
    Call ExecTool(CmdLine)
  Else
    ' we need to run seaudit three times
    CmdLine = SeAuditTool + ConstCmdSwitch + start1+stop1 + OutPutcatcher
    Call ExecTool(CmdLine)
    ' all days in between
    start2=" -sd "+ sd_plus_one
    stop2=" -ed "+ ed_minus_one
    CmdLine = SeAuditTool + ConstCmdSwitch + start2+stop2 + OutPutcatcher2
    Call ExecTool(CmdLine)
    CmdLine = SeAuditTool + ConstCmdSwitch + start3+stop3 + OutPutcatcher2
    Call ExecTool(CmdLine)
  End If
End If

wshShell.RegWrite regKey, FormatDateTime(enddatetime, 0)

If Err.Number <> 0 Then
	Call Debug("Unable to save last collect date (" + enddatetime + ") to register!")
End If

Call CleanExit(0)

'=============================
'
' Sub : getDate (in dd-mmm-yyyy format)
'
Sub getDate(ByVal myDate, ByRef theDate)
  theDate =        Day(myDate) _
           & "-" & MonthName(Month(myDate),True) _
           & "-" & Year(myDate)
End Sub

'=============================
'
' Sub : getTime (in hh:mm 24-hrs notation)
'
Sub getTime(ByVal myTime, ByRef theTime)
  If Minute(myTime)<10 Then
   theTime =  Hour(myTime) _
            & ":0" & Minute(myTime)
  Else
   theTime =  Hour(myTime) _
            & ":" & Minute(myTime)
  End If
End Sub

'============================
'
' infoMessage()
'
Sub infoMessage(outputFile, text)
  If tracing Then
    wscript.echo "Message: " & text
  End If

  Call outputFile.WriteLine(Now & " " & text)
End Sub

'============================
'
' DeleteAFile(filename)
'
Sub DeleteAFile(filespec)
  Dim fso
  Set fso = CreateObject("Scripting.FileSystemObject")
  fso.DeleteFile(filespec)
End Sub

'============================
'
' Debug(message)
'
Sub Debug(ByVal message)
  If Tracing Then
    Call infoMessage(debugLogFile,message)
  End If
End Sub

'============================
'
' CleanExit(error_code)
'
Sub CleanExit(error_code)
	On Error Resume Next
	Call debugLogFile.Close()
	If Not tracing Then
	  Call DeleteAFile(debugLog)
	End If
	Call Debug("Exiting with error code " & error_code)
	wscript.Quit error_code
End Sub

'============================
'
' ExecTool(cmd_line)
'
Sub ExecTool(cmd_line)
  On Error Resume Next
  Call Debug("Executing: " + cmd_line)
  returnCode = wshShell.Run(cmd_line, 0, True)
  If returnCode <> 0 Then
  	Call Debug("Execution failed!")
  	Call CleanExit(returnCode)
  End If
End Sub

