'***********************************************************BeginFile************
' Copyright:  (c) 2000 by Consul Risk Management B.V.  All Rights Reserved
'
' Time-stamp: <010725 Ide 17:15 getfwlog.vbs>
'
' Name:
'   getfwlog.vbs
'
' Purpose:
'   retrieve log entries since the previous run.
'   and store them sequentially in a new file
'
' Notes:
'   Syntax:
'     getfwlog.vbs <outfile> [<path to FireWall-1 software>]
'
'   <outfile> should be something like ".\firewall.log"
'   <path to FireWall-1 software> defaults to "c:\winnt\fw1\4.1"
'   The FireWall-1 log file will receive a name with the following format:
'     <year><month><day><hour><minute><second>.log
'   the year in four digits, others in two digits
'
' History:
'   000613 0.0.1   HJvT  EE003006:  Created.
'   010725         IN    added '-n' switch
'   VL 2004sep29 updated: QE0309001: Cannot collect FW1 NG log using agent.
'                         ".\" string has been prepended to the name of
'                         the temporary file that is produced by the script
'***********************************************************EndFile**************
Option Explicit      ' Force explicit variable declaration.
On Error GoTo 0


' Variable declaration

Dim args             ' The arguments of this program
Dim outFileName      ' The file containing the collected log data
Dim fso              ' Temporary File System Object to check folder existence
Dim fw1Dir           ' Path to FireWall-1 software
Dim fw               ' The FireWall-1 Command
Dim thisMoment       ' The current time
Dim inFileName       ' File to translate to readable format by fw
Dim returnCode       ' Result of a command
Dim debugLogObject
Dim debugLogFile
Dim WshShell         ' The shell to direct commands to
Dim newAuditLogFileName
Dim logExportCommand ' The FireWall-1 command to export a file in readable format
Dim logSwitchCommand ' The FireWall-1 command to switch log files
Dim switchedLogFile  ' The file created by fw after receiving the logswitch command

' Constant declaration
Const debugLog = "..\log\fw1Debug.Log"
Const tracing  = False ' If True, tracing messages are written to stdout

' Initiate logging file
Set debugLogObject = CreateObject("Scripting.FileSystemObject")
Set debugLogFile   = debugLogObject.CreateTextFile(debugLog, True)

If tracing Then
  wscript.echo "Opened log file " & debugLog
End If

' Parameter handling

Set args = wscript.Arguments

' Check whether "Temporary File 1" is specified
If args.Count = 0 Then
  Call errorMessage(debugLogFile, "ERROR: 'Temporary File 1' is not specified")
  Call wscript.Quit(1)
End If

If args.Count > 2 Then
  If args(2) <> " " And args(2) <> "" Then
    Call errorMessage(debugLogFile, "ERROR: Too many properties specified")
    Call wscript.Quit(1)
  End If
End If

' Get properties as passed by ExtendIT toolkit.
outFileName = args(0)      ' TEMPFILE1

fw1Dir = "c:\winnt\fw1\4.1"
If args.Count > 1 Then
  If args(1) <> " " And args(1) <> "" Then
    fw1Dir = args(1)       ' OPTION1
  End If
End If

If tracing Then
  wscript.echo "outFileName: " & outFileName
  wscript.echo "fw1Dir:      " & fw1Dir
End If

' Does the path to the FireWall-1 software really exist?
Set fso = CreateObject("Scripting.FileSystemObject")
If (fso.FolderExists(fw1Dir) = False) Then
  Call errorMessage(debugLogFile, "ERROR: The path to the FireWall-1 software (" & fw1Dir & ") does not exist")
  Call wscript.Quit(1)
End If

' The FireWall-1 Command
fw = fw1Dir & "\bin\fw"

Call infoMessage(debugLogFile, "Temporary File 1: " & outFileName)

'
' Tell the firewall to switch it's log
'
Err.Clear

Set WshShell        = Wscript.CreateObject("Wscript.Shell")
thisMoment          = Now
newAuditLogFileName = Year(thisMoment)                     _
                      & Right("0" & Month(thisMoment),  2) _
                      & Right("0" & Day(thisMoment),    2) _
                      & Right("0" & Hour(thisMoment),   2) _
                      & Right("0" & Minute(thisMoment), 2) _
                      & Right("0" & Second(thisMoment), 2)

switchedLogFile     = fw1Dir & "\log\" & newAuditLogFileName & ".log"

logSwitchCommand = fw & " logswitch " & newAuditLogFileName
Call infoMessage(debugLogFile, logSwitchCommand)
returnCode = WshShell.Run(logSwitchCommand, 0, True)

If (fso.FileExists(switchedLogFile) = False) Then
  Call errorMessage(debugLogFile, _
                    "ERROR: the fw command failed, while switching logs; the expected file " _
                    & switchedLogFile & " was not found")
Else
  '
  ' Tell the firewall to export it's log
  '
  ' 2004sep29VL QE0309001 changed/ ".\" string has been prepended to the name of the output file
  logExportCommand = fw & " logexport -n -i  " & switchedLogFile & " -o .\" & outFileName
  returnCode       = WshShell.Run(logExportCommand, 0, True)

  If (fso.FileExists(outFileName) = False) Then
    Call errorMessage(debugLogFile, _
                      "ERROR: the fw command failed, while exporting; the expected file " _
                      & outFileName & " was not found")
  End If
End If

If tracing Then
  wscript.echo "Closing and quitting"
End If

Call debugLogFile.Close()
Call wscript.Quit(returnCode)
'============================

'
' infoMessage()
'
Sub infoMessage(outputFile, text)
  If tracing Then
    wscript.echo "Message: " & text
  End If

  Call outputFile.WriteLine(Now & " " & text)
End Sub


'
' erorMessage()
'
Sub errorMessage(outputFile, text)
  Call infoMessage(outputFile, text)
End Sub

'/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
