'***********************************************************************
' Copyright:  (c) 1998 by Consul Risk Management B.V.  All Rights Reserved
'
' Time-stamp: <030804 ide 17:29 getsmlnlog.vbs>
' Name:
'    getsmlnlog.vbs
'
' Purpose:
'   Read ScanMail log file smvlog.nsf from server (2nd argument) and write information
'   into output file (1st argument)
' Notes:
'   getsmlnlog.vbs <outfile> <server> <databasefile> <userid> <pwd> <dummmy> <propsfile>
'
'   If <userid> is entered, then the script MUST be run at the Lotus Domino Server
'
'   <pwd> may not be empty, as server connetion is no longer supported
'
' History:
'    01/12/2002 : created by Elena Shutova
'    02/10/2002 : edited by Elena Shutova
'    05/30/2002 : edited by Elena Shutova
'       update last data access into the registry
'    030626 <version> ide <ID>: extended 
'    040613 AndrewZhdanov: fixes:
'           - separate log and regkeys for each event source
'           - do not allow empty password - no server connection
'           - fix NotesSession initializing to run on Lotus Notes
'           - fail graicfully if corresponding view or document property does not exist
'                       
'***********************************************************************
Option Explicit
On Error Resume Next ' handle error with handleError Sub

Dim args             ' Arguments parsed
Dim OutFileName      ' Name of file to copy misc. events to
Dim nlServer         ' Name of the Lotus Notes Server
Dim GetDbnlServer    ' Real Servername   
Dim nlDatabaseFile   ' databasefile name of events (log.nsf)
Dim nlUserId         ' User used to log in to notes Server
Dim nlPwd            ' Password of nlUserId
Dim Dummy            ' Reserved for future needs
Dim PropsFile        ' Unique Id of Event Source instance
Dim debugLogObject   ' DebugLog
Dim debugLogFile     ' Enable debuggin messages ?
Dim notesVersion     ' Notes Version number
Dim errorNum         ' Error number 
Dim errorDesc        ' Error Description

' Constants
Const debugLog = "..\log\ScanMailLN_"
Const tracing  = False

' Command Line argument pasring
Set args = wscript.Arguments
OutFileName     = args(0)
nlServer        = args(1)
nlDatabaseFile  = args(2)
nlUserId        = args(3)
nlPwd           = args(4)
Dummy           = args(5)
PropsFile       = args(6)

' Adjust variables
If nlServer = " " Then nlServer = ""       
If nlPwd    = " " Then nlPwd = ""
If nlUserId = " " Then nlUserId = ""

Dim regkey, regkey_old, regkeyMail, nlViewName, nlViewNameMail
regkey_old = "HKLM\Software\CONSUL Risk Management\CEAActuator\" & "SMLN" _
         & "\" & nlServer & "-" & nlDatabaseFile
regkey = "HKLM\Software\CONSUL Risk Management\CEAActuator\" _
         & "SMLN_" & PropsFile _
         & "\" & nlServer & "-" & nlDatabaseFile
nlViewName = "by Date"

' Initiate logging file and write arguments
Dim removedProps, append
removedProps = Replace(PropsFile, ".props", "")
append = debugLog & removedProps & ".log"
Set debugLogObject = CreateObject("Scripting.FileSystemObject")
Set debugLogFile   = debugLogObject.CreateTextFile(append, True)
If tracing Then
  wscript.echo "Opened log file " & debugLog
  Call infoMessage(debugLogFile, "Open ScanMail for Lotus Notes Logfile")
  Call infoMessage(debugLogFile, "OutFileName     = args(0) : " & OutFileName )
  Call infoMessage(debugLogFile, "nlServer        = args(1) : " & nlServer )
  Call infoMessage(debugLogFile, "nlDatabaseFile  = args(2) : " & nlDatabaseFile )
  Call infoMessage(debugLogFile, "nlUserId        = args(3) : " & nlUserId )
  If nlPwd = "" Then 
    Call infoMessage(debugLogFile, "nlPwd           = args(4) : " & "<empty>" )
  Else
    Call infoMessage(debugLogFile, "nlPwd           = args(4) : " & "****" )
  End If
  Call infoMessage(debugLogFile, "Dummy           = args(5) : " & Dummy )
  Call infoMessage(debugLogFile, "PropfFile       = args(6) : " & PropsFile )
End If

' Validate input
If nlPwd = "" Then
  Call handleError(-1, "Password can not be empty, as server connection is no longer supported")
End If

' Initiate wscript shell
Dim wshShell
Call infoMessage(debugLogFile, "Creating shell")
Set wshShell = wscript.CreateObject("WScript.Shell")
errorNum  = Err.Number
errorDesc = Err.Description
Call handleError(errorNum, errorDesc )

'Open a new Notes session and open database
On Error Resume Next
Call infoMessage(debugLogFile, "Creating Lotus.NotesSession Object")
Dim session, logDb
Set session = CreateObject("Lotus.NotesSession")
errorNum  = Err.Number
errorDesc = Err.Description
Call handleError(errorNum, errorDesc )
Call infoMessage(debugLogFile, "Lotus.NotesSession Object created.")

'Initializing Session
' For help, see Notes 5 Administration Help, item : "hosting java applets"
Call infoMessage(debugLogFile, "Initializing Session")
If nlUserId = "" Then
  Call infoMessage(debugLogFile, " - Initializing with current user ID")
  session.Initialize nlPwd
  
  ' Initialize: this method can be used on a computer with a Notes client or
  ' Domino server and bases the session on the current user ID. If a password
  ' is specified, it must match the user ID password. If a password is not
  ' specified, the user is prompted for a password as required and as the
  ' software permits. Since VBScript does not support prompting, you must
  ' supply the password or the user ID must not have one.
  
Else
  Call infoMessage(debugLogFile, " - Initializing with user " & nlUserId)
  session.InitializeUsingNotesUserName nlUserId, nlPwd
  
  ' InitializeUsingNotesUserName: this method can be used only on a computer
  ' with a Domino server. InitializeUsingNotesUserName looks the name up in the
  ' local Domino Directory and permits access to the local server depending on
  ' the "Server Access" and "COM Restrictions" settings. The password must match
  ' the Internet password associated with the name. If no name is specified,
  ' access is granted if the server permits Anonymous access.
  
End If
errorNum  = Err.Number
errorDesc = Err.Description
Call handleError(errorNum, errorDesc )
Call infoMessage(debugLogFile, " - Initialization succeeded")

' Opening database on Server
Call infoMessage(debugLogFile, "Opening database " & nlDatabaseFile)
If nlServer = "Local" Then
  Call infoMessage(debugLogFile, " - Opening database on local server")
  GetDbnlServer = ""
Else
  Call infoMessage(debugLogFile, " - Opening database on server " & nlServer)
  GetDbnlServer = nlServer
End If  
Set logDb = session.GetDatabase(GetDbnlServer, nlDatabaseFile)
errorNum  = Err.Number
errorDesc = Err.Description
Call handleError(errorNum, errorDesc )

' The logDb.Open call throws an error if the DB is already open. Ignore this error
' and continue
logDb.Open
If logDb.IsOpen Then
  Call infoMessage(debugLogFile, " - Opening database succeeded")
  ' Read Miscellaneous Events
  Call infoMessage(debugLogFile, "")
  Call infoMessage(debugLogFile, "* Reading ScanMail database")
  GetLog logDb, OutFileName, nlViewName, regkey, regkey_old
  Call infoMessage(debugLogFile, "Prossesing Event completed successfully")
Else
  Call handleError(-1, "Database file not found")
End If


' Close debugfile and exit
Call debugLogFile.Close()
wscript.quit

'*************************************************************************************
' Name: GetLog
' Purpose: Copy events from a view in an opened database to a file
' Parameters:
'   logDb: An open NotesDatabase
'   OutFileName:              Name of file to copy events to.
'   nlViewName:               Name of view within Lotus.
'   regkey:                   Registry subkey to store timestamp between runs
'**************************************************************************************


'Get log from SMVLOG.NSF database (log for MailScan results)
Sub GetLog(logDb, OutFileName, nlViewName, regkey, regkey_old)

  Dim LastStartTimePreviousRun, StartTime, LastStartTime, ProcessStartTime, StartTm
  Dim AllDocsView, AllDocsViewNav, CurrentNavEntry, Doc, ProcessDoc
  Dim ProcessDocs(), LastDocs()
  Dim iStart, iEnd, FinishTime
  Dim LogTime,LogDate
  Dim LogServer,LogUser,LogEvent, LogFrom, LogTo
  Dim LogInfected,LogVirusName,LogAction,LogScanner,LogScanProg
  Dim LogString
  Dim event_cnt
  Dim fso, MyFile

  Call infoMessage(debugLogFile, "Opening View " & nlViewName)
  
  'Initialize tampstamp-keeping
  On Error Resume Next
  LastStartTimePreviousRun = wshShell.RegRead(regkey)
  On Error Goto 0

  If Len(LastStartTimePreviousRun) = 0 Then
    On Error Resume Next
    LastStartTimePreviousRun = wshShell.RegRead(regkey_old)
    On Error Goto 0
    If Len(LastStartTimePreviousRun) = 0 Then LastStartTimePreviousRun = "01/01/1970 00:00:00"
  End If
  
  Call infoMessage(debugLogFile, "Last Collect started at " & LastStartTimePreviousRun)

  'Create the output file
  Call infoMessage(debugLogFile, "Creating outputfile " & OutFileName)
  Set fso = CreateObject("Scripting.FileSystemObject")
  Set MyFile = fso.CreateTextFile(OutFileName, True)
  Call infoMessage(debugLogFile, "- outputfile created ")

  'Open a new view with all documents
  Call infoMessage(debugLogFile, "Retrieving all documents in view")
  Set AllDocsView = logDb.GetView(nlViewName)
  If AllDocsView Is Nothing Then Call handleError(-1, "View not found")
  Set AllDocsViewNav = AllDocsView.CreateViewNav()
  Set CurrentNavEntry = AllDocsViewNav.GetFirstDocument
  Call infoMessage(debugLogFile, "- Retrieved first document")
  event_cnt = 0

  Do While Not (CurrentNavEntry Is Nothing)
    Set Doc = CurrentNavEntry.Document
    ' log entry in debugfile
    On Error Resume Next
    Call infoMessage(debugLogFile, "- Retrieved document " & Doc.NotesURL )
    On Error GoTo 0
    
    If Not (Doc Is Nothing) Then
        StartTime = Doc.GetFirstItem("StartTime").Text 
        On Error Resume Next
        StartTm = Doc.GetFirstItem("I_Zeit").Text
        StartTime = StartTime & " " & StartTm
      If CDate(StartTime) >= CDate(LastStartTimePreviousRun) Then
        'Do not process last document(s) because they may not be complete yet
        If CDate(StartTime) >= CDate(LastStartTime) Then
          ProcessStartTime = LastStartTime
          CopyObjArray LastDocs, ProcessDocs
          LastStartTime = StartTime
          ReDim LastDocs(0)
          Set LastDocs(0) = Doc
        'ElseIf (CDate(StartTime) = CDate(LastStartTime)) Then
        '  ReDim Preserve LastDocs(ArrayDim(LastDocs))
        '  Set LastDocs(ArrayDim(LastDocs)-1) = Doc
        '  Erase ProcessDocs
        Else 'CDate(StartTime) < CDate(LastStartTime)
          'LastStartTime = StartTime  ' IN2003aug04: no laststarttime in starttime < laststarttime
          ProcessStartTime = StartTime
          ReDim ProcessDocs(0)
          Set ProcessDocs(0) = Doc
        End If
        Call infoMessage(debugLogFile, "- StartTime = " & StartTime)
        
        For Each ProcessDoc in ProcessDocs
          Call infoMessage(debugLogFile, "  - ProcessStartTime = " & ProcessStartTime)

          event_cnt = event_cnt+1
          'Month(monat)/Day(tag)/Year(jahr)
          LogDate = ProcessDoc.GetFirstItem("I_Monat").Text & "/"
          LogDate = LogDate & ProcessDoc.GetFirstItem("I_Tag").Text & "/"
          LogDate = LogDate & ProcessDoc.GetFirstItem("I_Jahr").Text
          LogTime = ProcessDoc.GetFirstItem("I_Zeit").Text
          LogString = LogString & "#Date: " & LogDate & " " & LogTime

          'From
          LogFrom = ProcessDoc.GetFirstItem("Server").Text
          LogString = LogString & " " & "#From: " & LogFrom
          LogTo = ProcessDoc.GetFirstItem("I_Empfaenger").Text
          LogString = LogString & " " & "#To: " & LogTo

          'UserName
          LogUser = ProcessDoc.GetFirstItem("I_UserName").Text
          LogString = LogString & " " & "#UserName: " & LogUser
          
          'Infected file name
          LogInfected =  ProcessDoc.GetFirstItem("I_Infectfilename").Text
          LogString = LogString & " " & "#Infected file: " & LogInfected
          
          LogVirusName = "NO"
          'If RTrim(LTrim(LogInfected)) <> "Encrypted Document" Then
          On Error Resume Next
          'Infected file name
          LogVirusName =  ProcessDoc.GetFirstItem("I_VirusName").Text
          'End If
          LogString = LogString & " " & "#Virus Name: " & LogVirusName

          'Action
          LogAction = ProcessDoc.GetFirstItem("I_Aktion").Text
          LogString = LogString & " " & "#Action: " & LogAction

          'Scanner
          LogScanner = ProcessDoc.GetFirstItem("Scanner").Text
          LogString = LogString & " " & "#Scanner: " & LogScanner

          'Scanning
          LogScanProg = ProcessDoc.GetFirstItem("I_Scanprog").Text
          LogString = LogString & " " & "#Scanning: " & LogScanProg

          'Events
          LogEvent = ProcessDoc.GetFirstItem("Events").Text       
          LogString = LogString & " " & "#Events: " & LogEvent

          LogString = LogString & " " & "#UpdatedBy: " & ProcessDoc.GetFirstItem("$UpdatedBy").Text

          FinishTime = ProcessDoc.GetFirstItem("FinishTime").Text
          LogString = LogString  & " " & "#FinishTime: " & FinishTime
          MyFile.WriteLine LogString
          LogString = ""

          On Error GoTo 0
          If Err.Number <> 0 Then Call handleError(-1, "Can't read document from the database")

        Next
      End If  
    Else
      Call infoMessage(debugLogFile, "  - Document is empty" )
    End If
   'Go to next document, if any      
    If Doc Is Nothing Then
      Set CurrentNavEntry = Nothing
    Else
      Set CurrentNavEntry = AllDocsViewNav.GetNextDocument(CurrentNavEntry)
    End If
  Loop
  Call infoMessage(debugLogFile, "- All Docs Processed, last time = " & LastStartTime)

  MyFile.WriteLine "Events: " & event_cnt
  MyFile.Close
  If LastStartTime <> "" Then wshShell.RegWrite regkey, LastStartTime

End Sub


'***************************************************
' Name: ArrayDim
' Purpose: Get the number of elements in an array
'***************************************************
Function ArrayDim(Arr)
  Dim Count
  On Error Resume Next
  Count = UBound(Arr) + 1
  ArrayDim = Count
End Function  

'****************************************************
' Name: CopyObjArray
' Purpose: Make a copy of an array containing objects
'****************************************************
Sub CopyObjArray(SourceArr, TargetArr)
  Dim Count, Index
  Count = ArrayDim(SourceArr)
  If Count = 0 Then
    Erase TargetArr
  Else
    ReDim TargetArr(Count - 1)
    For Index = 0 To Count - 1
      Set TargetArr(Index) = SourceArr(Index)
    Next
  End If
End Sub


'============================
'
' infoMessage()
'
Sub infoMessage(outputFile, text)
  If tracing Then
    wscript.echo "Message: " & text
  End If

  Call outputFile.WriteLine(Now & " " & text)
End Sub

'============================
'
' handleError()
'
Sub handleError(errorNum, errorDesc)
  If errorNum =0 Then
    'None
  Else
    Call infoMessage(debugLogFile, "ERROR Occured : " & errorDesc)
    wscript.quit errorNum
  End If
End Sub
